//This code was downloaded from https://github.com/P-N-Suganthan/2022-SO-BO/tree/main

//Modifications by R. Biedrzycki rbiedrzy@elka.pw.edu.pl, 2023:
//1) 10e-8 was changed to 1e-8
//2) all numeric constants were named and their values can be specified when starting the program

#include <cmath>
#include <iostream>
//#include <ctime>
#include <fstream>
#include <random>
#include <boost/program_options.hpp>

#include "cec22_test_func.cpp"


//g++ nl_shade_lbc.cpp -lboost_program_options

//default configuration: ./a.out --seed 1234 --res_file_name default --pop_size_mult 23 --mem_size_mult 20 --arch_size_mult 1 --initial_CR 0.9 --initial_F 0.5 --min_pop_size 4 --arch_prob 0.5 --per_of_the_best_considered_at_start 20 --per_of_the_best_considered_at_end 30 --norm_sigma 0.1 --cauchy_sigma 0.1 --default_CR 0.5 --default_F 0.5 --MWLp1 3.5 --MWLp2 1 --MWLm 1.5 --LBC_fin 1.5

using namespace std;
namespace po = boost::program_options;

std::mt19937 generator;
std::uniform_int_distribution<int> uni_int(0,32768);
std::uniform_real_distribution<double> uni_real(0.0,1.0);
std::normal_distribution<double> norm_dist(0.0,1.0);
std::cauchy_distribution<double> cachy_dist(0.0,1.0);

int IntRandom(int target)
{
    if(target == 0)
        return 0;
    return uni_int(generator)%target;
}
double Random(double minimal, double maximal){return uni_real(generator)*(maximal-minimal)+minimal;}
double NormRand(double mu, double sigma){return norm_dist(generator)*sigma + mu;}
double CachyRand(double mu, double sigma){return cachy_dist(generator)*sigma+mu;}

void qSort1(double* Mass, int low, int high)
{
    int i=low;
    int j=high;
    double x=Mass[(low+high)>>1];
    do
    {
        while(Mass[i]<x)    ++i;
        while(Mass[j]>x)    --j;
        if(i<=j)
        {
            double temp=Mass[i];
            Mass[i]=Mass[j];
            Mass[j]=temp;
            i++;    j--;
        }
    } while(i<=j);
    if(low<j)   qSort1(Mass,low,j);
    if(i<high)  qSort1(Mass,i,high);
}
void qSort2int(double* Mass, int* Mass2, int low, int high)
{
    int i=low;
    int j=high;
    double x=Mass[(low+high)>>1];
    do
    {
        while(Mass[i]<x)    ++i;
        while(Mass[j]>x)    --j;
        if(i<=j)
        {
            double temp=Mass[i];
            Mass[i]=Mass[j];
            Mass[j]=temp;
            int temp2=Mass2[i];
            Mass2[i]=Mass2[j];
            Mass2[j]=temp2;
            i++;    j--;
        }
    } while(i<=j);
    if(low<j)   qSort2int(Mass,Mass2,low,j);
    if(i<high)  qSort2int(Mass,Mass2,i,high);
}

void cec22_test_func(double *, double *,int,int,int);
double *OShift,*M,*y,*z,*x_bound;
int ini_flag=0,n_flag,func_flag,*SS;
int dim;
double stepsFEval[16];
//const int NMBR_OF_RUNS=5;
const int NMBR_OF_RUNS=30;
double ResultsArray[12][NMBR_OF_RUNS][17];
int LastFEcount;
int NFEval = 0;
int MaxFEval = 0;
double tempF[1];
double fopt;
char buffer[200];
double globalbest;
bool globalbestinit;
bool initfinished;
vector<double> FitTemp3;

void GenerateNextRandUnif(const int num, const int Range, int* Rands, const int Prohib)
{
    for(int j=0;j!=25;j++)
    {
        bool generateagain = false;
        Rands[num] = IntRandom(Range);
        for(int i=0;i!=num;i++)
            if(Rands[i] == Rands[num])
                generateagain = true;
        if(!generateagain)
            break;
    }
}
void GenerateNextRandUnifOnlyArch(const int num, const int Range, const int Range2, int* Rands, const int Prohib)
{
    for(int j=0;j!=25;j++)
    {
        bool generateagain = false;
        Rands[num] = IntRandom(Range2)+Range;
        for(int i=0;i!=num;i++)
            if(Rands[i] == Rands[num])
                generateagain = true;
        if(!generateagain)
            break;
    }
}
bool CheckGenerated(const int num, int* Rands, const int Prohib)
{
    if(Rands[num] == Prohib)
        return false;
    for(int j=0;j!=num;j++)
        if(Rands[j] == Rands[num])
            return false;
    return true;
}
void SaveBestValues(int funcN, int RunN, double newbestfit)
{
    double temp = globalbest - fopt;
    if(temp <= 1E-8 && ResultsArray[funcN-1][RunN][16] == MaxFEval)
    {
        ResultsArray[funcN-1][RunN][16] = NFEval;
    }
    for(int stepFEcount=LastFEcount;stepFEcount<16;stepFEcount++)
    {
        if(NFEval == int(stepsFEval[stepFEcount]*MaxFEval))
        {
            if(temp <= 1E-8)
                temp = 0;
            ResultsArray[funcN-1][RunN][stepFEcount] = temp;
            LastFEcount = stepFEcount;
        }
    }
}

void FindLimits(double* Ind, double* Parent,int CurNVars,double CurLeft, double CurRight)
{
    for (int j = 0; j<CurNVars; j++)
    {
        for (int j = 0; j<CurNVars; j++)
        {
            if (Ind[j] < CurLeft)
                Ind[j] = (CurLeft + Parent[j])/2.0;
            if (Ind[j] > CurRight)
                Ind[j] = (CurRight + Parent[j])/2.0;
        }
    }
}
class Optimizer
{
public:
    double initial_CR;
    double initial_F;
    int min_pop_size;
    float arch_prob;
    double per_of_the_best_considered_at_start;
    double per_of_the_best_considered_at_end;    
    double norm_sigma;
    double cauchy_sigma;
    float default_CR;
    float default_F;     


    bool FitNotCalculated;
    int Int_ArchiveSizeParam;
    int MemorySize;
    int MemoryIter;
    int SuccessFilled;
    int MemoryCurrentIndex;
    int NVars;
    int popSize;
    int NIndsMax;
    int NIndsMin;
    int besti;
    int func_num;
    int RunN;
    int Generation;
    int ArchiveSize;
    int CurrentArchiveSize;
    double MWLp1;
    double MWLp2;
    double MWLm;
    double LBC_fin;
    double F;
    double Cr;
    double bestfit;
    double ArchiveSizeParam;
    double Right;
    double Left;
    int* Rands;
    int* Indexes;
    int* BackIndexes;
    double* Weights;
    double* Donor;
    double* Trial;
    double* FitMass;
    double* FitMassTemp;
    double* FitMassCopy;
    double* BestInd;
    double* tempSuccessCr;
    double* tempSuccessF;
    double* FGenerated;
    double* CrGenerated;
    double* MemoryCr;
    double* MemoryF;
    double* FitDelta;
    double* FitMassArch;
    double** Popul;
    double** PopulTemp;
    double** Archive;

    void Initialize(int newPopSize, int newNVars, int new_func_num, int newRunN, int NewMemSize, double NewArchSizeParam, double initial_CR, double initial_F, int min_pop_size,
                    float arch_prob, float per_of_the_best_considered_at_start, float per_of_the_best_considered_at_end, float norm_sigma, float cauchy_sigma,
                    float default_CR, float default_F, float MWLp1, float MWLp2, float MWLm, float LBC_fin, const vector<vector<double>>& startPop);
    void Clean();
    void MainCycle();
    void FindNSaveBest(bool init, int ChosenOne);
    inline double GetValue(const int index, const int popSize, const int j);
    void CopyToArchive(double* RefusedParent,double RefusedFitness);
    void SaveSuccessCrF(double Cr, double F, double FitD);
    void UpdateMemoryCrF();
    double MeanWL_general(double* Vector, double* TempWeights, int Size, double g_p, double g_m);
    void RemoveWorst(int popSize, int NewNInds);
};

double cec_22_(double* HostVector,int func_num){
	cec22_test_func(HostVector, tempF, dim, 1, func_num);
    NFEval++;
    return tempF[0];
}

void Optimizer::Initialize(int newPopSize, int newNVars, int new_func_num, int newRunN,
                           int NewMemSize, double NewArchSizeParam, double initial_CR, double initial_F, int min_pop_size, float arch_prob,
                           float per_of_the_best_considered_at_start, float per_of_the_best_considered_at_end, float norm_sigma, float cauchy_sigma,
                           float default_CR, float default_F, float aMWLp1, float aMWLp2, float aMWLm, float aLBC_fin, const vector<vector<double>>& startPop){
    this->initial_CR=initial_CR;
    this->initial_F=initial_F;
    this->min_pop_size=min_pop_size;
    this->arch_prob=arch_prob;
    this->per_of_the_best_considered_at_start=per_of_the_best_considered_at_start;
    this->per_of_the_best_considered_at_end= per_of_the_best_considered_at_end;
    this->norm_sigma=norm_sigma;
    this->cauchy_sigma=cauchy_sigma;   
    this->default_CR=default_CR;
    this->default_F=default_F;


    FitNotCalculated = true;
    popSize = newPopSize;
    NIndsMax = popSize;
    NIndsMin = min_pop_size;
    NVars = newNVars;
    RunN = newRunN;
    Left = -100;
    Right = 100;
    Cr = initial_CR;//0.9;
    F = initial_F;//0.5;
    besti = 0;
    Generation = 0;
    CurrentArchiveSize = 0;
    
    MWLp1 = aMWLp1;
    MWLp2 = aMWLp2;
    MWLm = aMWLm;
    LBC_fin = aLBC_fin;

    ArchiveSizeParam = NewArchSizeParam;
    Int_ArchiveSizeParam = ceil(ArchiveSizeParam);
    ArchiveSize = NIndsMax*ArchiveSizeParam;
    func_num = new_func_num;
    for(int steps_k=0;steps_k!=15;steps_k++)
        stepsFEval[steps_k] = pow(double(dim),double(steps_k)/5.0-3.0);
    stepsFEval[15] = 1.0;
    Popul = new double*[NIndsMax];
    for(int i=0;i!=NIndsMax;i++)
        Popul[i] = new double[NVars];
    PopulTemp = new double*[NIndsMax];
    for(int i=0;i!=NIndsMax;i++)
        PopulTemp[i] = new double[NVars];
    Archive = new double*[NIndsMax*Int_ArchiveSizeParam];
    for(int i=0;i!=NIndsMax*Int_ArchiveSizeParam;i++)
        Archive[i] = new double[NVars];
    FitMass = new double[NIndsMax];
    FitMassTemp = new double[NIndsMax];
    FitMassCopy = new double[NIndsMax];
    FitMassArch = new double[NIndsMax*Int_ArchiveSizeParam];
    Indexes = new int[NIndsMax];
    BackIndexes = new int[NIndsMax];
    BestInd = new double[NVars];
	for (int i = 0; i<NIndsMax; i++)
		for (int j = 0; j<NVars; j++)
			Popul[i][j] = startPop[i][j];//Random(Left,Right);
    Donor = new double[NVars];
    Trial = new double[NVars];
    Rands = new int[NIndsMax];
    tempSuccessCr = new double[NIndsMax];
    tempSuccessF = new double[NIndsMax];
    FitDelta = new double[NIndsMax];
    FGenerated = new double[NIndsMax];
    CrGenerated = new double[NIndsMax];
    for(int i=0;i!=NIndsMax;i++)
    {
        tempSuccessCr[i] = 0;
        tempSuccessF[i] = 0;
    }
    MemorySize = NewMemSize;
    MemoryIter = 0;
    SuccessFilled = 0;
    Weights = new double[NIndsMax];
    MemoryCr = new double[MemorySize];
    MemoryF = new double[MemorySize];
    for(int i=0;i!=MemorySize;i++)
    {
        MemoryCr[i] = initial_CR;//0.9;
        MemoryF[i] = initial_F;//0.5;
    }
}
void Optimizer::SaveSuccessCrF(double Cr, double F, double FitD)
{
    tempSuccessCr[SuccessFilled] = Cr;
    tempSuccessF[SuccessFilled] = F;
    FitDelta[SuccessFilled] = FitD;
    SuccessFilled ++ ;
}
void Optimizer::UpdateMemoryCrF()
{
    if(SuccessFilled != 0)
    {
        double FMWL = LBC_fin + (MWLp1-LBC_fin)*double(MaxFEval-NFEval)/(double)MaxFEval;
        double CrMWL= LBC_fin + (MWLp2-LBC_fin)*double(MaxFEval-NFEval)/(double)MaxFEval;
        MemoryF[MemoryIter] = (MemoryF[MemoryIter] + MeanWL_general(tempSuccessF, FitDelta,SuccessFilled,FMWL, MWLm))*0.5;
        MemoryCr[MemoryIter]= (MemoryCr[MemoryIter]+ MeanWL_general(tempSuccessCr,FitDelta,SuccessFilled,CrMWL,MWLm))*0.5;
        MemoryIter++;
        if(MemoryIter >= MemorySize)
            MemoryIter = 0;
    }
    else
    {
        MemoryF[MemoryIter] = default_F;
        MemoryCr[MemoryIter] = default_CR;
    }
}
double Optimizer::MeanWL_general(double* Vector, double* TempWeights, int Size, double g_p, double g_m)
{
    double SumWeight = 0;
    double SumSquare = 0;
    double Sum = 0;
    for(int i=0;i!=SuccessFilled;i++)
        SumWeight += TempWeights[i];
    for(int i=0;i!=SuccessFilled;i++)
        Weights[i] = TempWeights[i]/SumWeight;
    for(int i=0;i!=SuccessFilled;i++)
        SumSquare += Weights[i]*pow(Vector[i],g_p);
    for(int i=0;i!=SuccessFilled;i++)
        Sum += Weights[i]*pow(Vector[i],g_p-g_m);
    if(fabs(Sum) > 0.000001)
        return SumSquare/Sum;
    else
        return 0.5;
}
void Optimizer::CopyToArchive(double* RefusedParent,double RefusedFitness)
{
    if(CurrentArchiveSize < ArchiveSize)
    {
        for(int i=0;i!=NVars;i++)
            Archive[CurrentArchiveSize][i] = RefusedParent[i];
        FitMassArch[CurrentArchiveSize] = RefusedFitness;
        CurrentArchiveSize++;
    }
    else if(ArchiveSize > 0)
    {
        int RandomNum = IntRandom(ArchiveSize);
        int counter = 0;
        while(FitMassArch[RandomNum] < RefusedFitness)
        {
            RandomNum = IntRandom(ArchiveSize);
            counter++;
            if(counter == ArchiveSize)
                break;
        }
        for(int i=0;i!=NVars;i++)
            Archive[RandomNum][i] = RefusedParent[i];
        FitMassArch[RandomNum] = RefusedFitness;
    }
}
void Optimizer::FindNSaveBest(bool init, int ChosenOne)
{
    if(FitMass[ChosenOne] <= bestfit || init)
    {
        bestfit = FitMass[ChosenOne];
        besti = ChosenOne;
        for(int j=0;j!=NVars;j++)
            BestInd[j] = Popul[besti][j];
    }
    if(bestfit < globalbest)
        globalbest = bestfit;
}
void Optimizer::RemoveWorst(int popSize, int NewNInds)
{
    int PointsToRemove = popSize - NewNInds;
    for(int L=0;L!=PointsToRemove;L++)
    {
        double WorstFit = FitMass[0];
        int WorstNum = 0;
        for(int i=1;i!=popSize;i++)
        {
            if(FitMass[i] > WorstFit)
            {
                WorstFit = FitMass[i];
                WorstNum = i;
            }
        }
        for(int i=WorstNum;i!=popSize-1;i++)
        {
            for(int j=0;j!=NVars;j++)
                Popul[i][j] = Popul[i+1][j];
            FitMass[i] = FitMass[i+1];
        }
    }
}
inline double Optimizer::GetValue(const int index, const int popSize, const int j)
{
    if(index < popSize)
        return Popul[index][j];
    return Archive[index-popSize][j];
}
void Optimizer::MainCycle()
{
    double ArchProbs = arch_prob;
    for(int TheChosenOne=0;TheChosenOne!=popSize;TheChosenOne++)
    {
        FitMass[TheChosenOne] = cec_22_(Popul[TheChosenOne],func_num);
        FindNSaveBest(TheChosenOne == 0, TheChosenOne);
        if(!globalbestinit || bestfit < globalbest)
        {
            globalbest = bestfit;
            globalbestinit = true;
        }
        SaveBestValues(func_num, RunN, bestfit);
    }
    do
    {
        double minfit = FitMass[0];
        double maxfit = FitMass[0];
        for(int i=0;i!=popSize;i++)
        {
            FitMassCopy[i] = FitMass[i];
            Indexes[i] = i;
            if(FitMass[i] >= maxfit)
                maxfit = FitMass[i];
            if(FitMass[i] <= minfit)
                minfit = FitMass[i];
        }
        if(minfit != maxfit)
            qSort2int(FitMassCopy,Indexes,0,popSize-1);
        for(int i=0;i!=popSize;i++)
            for(int j=0;j!=popSize;j++)
                if(i == Indexes[j])
                {
                    BackIndexes[i] = j;
                    break;
                }
        FitTemp3.resize(popSize);
        for(int i=0;i!=popSize;i++)
            FitTemp3[i] = exp(-double(i)/(double)popSize);
        std::discrete_distribution<int> ComponentSelector3(FitTemp3.begin(),FitTemp3.end());
        //int psizeval = max(2.0,popSize*(0.1/(double)MaxFEval*(double)NFEval+0.2));
        double perTmp = per_of_the_best_considered_at_end-per_of_the_best_considered_at_start;
        if(perTmp<0)
            perTmp=0;
        int psizeval = max(2.0,popSize*(perTmp/(double)(MaxFEval)*(double)(NFEval)+per_of_the_best_considered_at_start));
        for(int TheChosenOne=0;TheChosenOne!=popSize;TheChosenOne++)
        {
            MemoryCurrentIndex = IntRandom(MemorySize);
            Cr = min(1.0,max(0.0,NormRand(MemoryCr[MemoryCurrentIndex], norm_sigma)));
            do
            {
                F = CachyRand(MemoryF[MemoryCurrentIndex], cauchy_sigma);
            }
            while(F <= 0);
            FGenerated[TheChosenOne] = min(F,1.0);
            CrGenerated[TheChosenOne] = Cr;
        }
        qSort1(CrGenerated,0,popSize-1);
        for(int TheChosenOne=0;TheChosenOne!=popSize;TheChosenOne++)
        {
            for(int Repeat = 0;Repeat != 100; Repeat++)
            {
                if(Repeat > 0)
                {
                    do
                    {
                        F = CachyRand(MemoryF[MemoryCurrentIndex],0.1);
                    }
                    while(F <= 0);
                    FGenerated[TheChosenOne] = min(F,1.0);
                }
                Rands[0] = Indexes[IntRandom(psizeval)];
                for(int i=0;i!=25 && !CheckGenerated(0,Rands,TheChosenOne);i++)
                    Rands[0] = Indexes[IntRandom(psizeval)];
                GenerateNextRandUnif(1,popSize,Rands,TheChosenOne);
                if(Random(0,1) > ArchProbs || CurrentArchiveSize == 0)
                {
                    Rands[2] = Indexes[ComponentSelector3(generator)];
                    for(int i=0;i!=25 && !CheckGenerated(2,Rands,TheChosenOne);i++)
                        Rands[2] = Indexes[ComponentSelector3(generator)];
                }
                else
                    GenerateNextRandUnifOnlyArch(2,popSize,CurrentArchiveSize,Rands,TheChosenOne);
                F = FGenerated[TheChosenOne];
                for(int j=0;j!=NVars;j++)
                    Donor[j] = Popul[TheChosenOne][j] +
                        FGenerated[TheChosenOne]*(GetValue(Rands[0],popSize,j) - Popul[TheChosenOne][j]) +
                        FGenerated[TheChosenOne]*(GetValue(Rands[1],popSize,j) - GetValue(Rands[2],popSize,j));

                int WillCrossover = IntRandom(NVars);
                Cr = CrGenerated[BackIndexes[TheChosenOne]];
                for(int j=0;j!=NVars;j++)
                {
                    if(Random(0,1) < Cr || WillCrossover == j)
                        PopulTemp[TheChosenOne][j] = Donor[j];
                    else
                        PopulTemp[TheChosenOne][j] = Popul[TheChosenOne][j];
                }
                bool stopRep = true;
                for(int j=0;j!=dim;j++)
                {
                    if(PopulTemp[TheChosenOne][j] > Right)
                        stopRep = false;
                    if(PopulTemp[TheChosenOne][j] < Left)
                        stopRep = false;
                }
                if(stopRep)
                    break;
            }
            FindLimits(PopulTemp[TheChosenOne],Popul[TheChosenOne],NVars,Left,Right);
            FitMassTemp[TheChosenOne] = cec_22_(PopulTemp[TheChosenOne],func_num);
            if(FitMassTemp[TheChosenOne] <= globalbest)
                globalbest = FitMassTemp[TheChosenOne];

            if(FitMassTemp[TheChosenOne] < FitMass[TheChosenOne])
                SaveSuccessCrF(Cr,F,fabs(FitMass[TheChosenOne]-FitMassTemp[TheChosenOne]));
            FindNSaveBest(false,TheChosenOne);
            SaveBestValues(func_num,RunN,bestfit);
        }
        for(int TheChosenOne=0;TheChosenOne!=popSize;TheChosenOne++)
        {
            if(FitMassTemp[TheChosenOne] <= FitMass[TheChosenOne])
            {
                CopyToArchive(Popul[TheChosenOne],FitMass[TheChosenOne]);
                for(int j=0;j!=NVars;j++)
                    Popul[TheChosenOne][j] = PopulTemp[TheChosenOne][j];
                FitMass[TheChosenOne] = FitMassTemp[TheChosenOne];
            }
        }
        int newPopSize = round((NIndsMin-NIndsMax)*pow((double(NFEval)/double(MaxFEval)),(1.0-double(NFEval)/double(MaxFEval)))+NIndsMax);
        if(newPopSize < NIndsMin)
            newPopSize = NIndsMin;
        if(newPopSize > NIndsMax)
            newPopSize = NIndsMax;
        int newArchSize = round((NIndsMin-NIndsMax)*pow((double(NFEval)/double(MaxFEval)),(1.0-double(NFEval)/double(MaxFEval)))+NIndsMax)*ArchiveSizeParam;
        if(newArchSize < NIndsMin)
            newArchSize = NIndsMin;
        if(newArchSize>NIndsMax*Int_ArchiveSizeParam)
          newArchSize=NIndsMax*Int_ArchiveSizeParam;            
        ArchiveSize = newArchSize;
        if(CurrentArchiveSize >= ArchiveSize)
            CurrentArchiveSize = ArchiveSize;
        RemoveWorst(popSize,newPopSize);
        popSize = newPopSize;
        UpdateMemoryCrF();
        SuccessFilled = 0;
        Generation ++;
    } while(NFEval < MaxFEval);
}
void Optimizer::Clean()
{
    delete Donor;
    delete Trial;
    delete Rands;
    for(int i=0;i!=NIndsMax;i++)
    {
        delete Popul[i];
        delete PopulTemp[i];
    }
    for(int i=0;i!=NIndsMax*Int_ArchiveSizeParam;i++)
        delete Archive[i];
    delete Archive;
    delete Popul;
    delete PopulTemp;
    delete FitMass;
    delete FitMassTemp;
    delete FitMassCopy;
    delete BestInd;
    delete Indexes;
    delete BackIndexes;
    delete tempSuccessCr;
    delete tempSuccessF;
    delete FGenerated;
    delete CrGenerated;
    delete FitDelta;
    delete MemoryCr;
    delete MemoryF;
    delete Weights;
}


  
int main(int argc, char* argv[]){
    po::options_description desc("Allowed options");
    unsigned int seed;
    double pop_size_mult=23.0;
    int mem_size_mult=20;
    float arch_size_mult=1;
    float initial_CR=0.9;
    float initial_F=0.5;
    int min_pop_size=4;
    float arch_prob = 0.5;
    float per_of_the_best_considered_at_start=20;
    float per_of_the_best_considered_at_end=30;
    float norm_sigma=0.1;
    float cauchy_sigma=0.1;
    float default_CR=0.5;
    float default_F=0.5; 
    float MWLp1 = 3.5;
    float MWLp2 = 1.0;
    float MWLm = 1.5;
    float LBC_fin = 1.5;

    std::string res_file_name;

    desc.add_options ()
        ("help", "print usage message")
        ("seed", po::value(&seed)->required(), "seed")
        ("pop_size_mult", po::value(&pop_size_mult), "pop_size_multiplier")
        ("mem_size_mult", po::value(&mem_size_mult), "memory size multiplier")
        ("arch_size_mult", po::value(&arch_size_mult), "archive size multiplier")
        ("initial_CR", po::value(&initial_CR), "initial_CR")
        ("initial_F", po::value(&initial_F), "initial_F")
        ("min_pop_size", po::value(&min_pop_size), "min_pop_size")
        ("arch_prob", po::value(&arch_prob), "prob of using archive")
        ("MWLp1", po::value(&MWLp1), "MWLp1")
        ("MWLp2", po::value(&MWLp2), "MWLp2")
        ("MWLm", po::value(&MWLm), "MWLm")
        ("LBC_fin", po::value(&LBC_fin), "LBC_fin")
        ("per_of_the_best_considered_at_start", po::value(&per_of_the_best_considered_at_start), "per_of_the_best_considered_at_start")
        ("per_of_the_best_considered_at_end", po::value(&per_of_the_best_considered_at_end), "per_of_the_best_considered_at_end")
        ("norm_sigma", po::value(&norm_sigma), "norm_sigma")
        ("cauchy_sigma", po::value(&cauchy_sigma), "cauchy_sigma")
        ("default_CR", po::value(&default_CR), "default_CR")
        ("default_F", po::value(&default_F), "default_F")
        ("res_file_name", po::value<std::string>(&res_file_name)->required(), "result file base name")    
        ; 
    po::variables_map vm;
    po::store (po::command_line_parser (argc, argv).options (desc).run (), vm);
    if(vm.count("help")){
        std::cout << desc << "\n";
        return 0;
    }

    po::notify(vm);
    generator.seed(seed);
    per_of_the_best_considered_at_start/=100;
    per_of_the_best_considered_at_end/=100;
    for(int GNVarsIter = 0;GNVarsIter!=2;GNVarsIter++)
    {
        if(GNVarsIter == 0)
        {
            dim = 10;
            MaxFEval = 200000;
        }
        if(GNVarsIter == 1)
        {
            dim = 20;
            MaxFEval = 1000000;
        }
        int POP_SIZE=dim*pop_size_mult;
        for (int RunN = 0;RunN!=NMBR_OF_RUNS;RunN++)
        {
            for(int func_num = 1; func_num < 13; func_num++)
            {
                string startPopFilename="../startPointsGen/startingPoints";
                startPopFilename+=to_string(dim)+'_'+to_string(RunN)+'_'+to_string(func_num)+".txt";
                ifstream fin(startPopFilename.c_str());
                vector<vector<double>> startPop;
                if(fin.is_open()){
                    double f;
                    char c;
                    for(int numOfStartPoints = 0;numOfStartPoints<POP_SIZE; ++numOfStartPoints){
                        vector<double> startInd;
                        for(int curDim = 0;curDim<dim; ++curDim){
                            fin>>f;
                            fin>>c;
                            startInd.push_back(f);
                        }
                        startPop.emplace_back(startInd);
                    }
                }else{
                    cout<<startPopFilename<<" open failed"<<endl;
                    return -1;
                }                


                ResultsArray[func_num-1][RunN][16] = MaxFEval;
                fopt = 0;
                switch(func_num)
                {
                    case 1:{fopt = 300;  break;}
                    case 2:{fopt = 400;  break;}
                    case 3:{fopt = 600;  break;}
                    case 4:{fopt = 800;  break;}
                    case 5:{fopt = 900;  break;}
                    case 6:{fopt = 1800; break;}
                    case 7:{fopt = 2000; break;}
                    case 8:{fopt = 2200; break;}
                    case 9:{fopt = 2300; break;}
                    case 10:{fopt = 2400;break;}
                    case 11:{fopt = 2600;break;}
                    case 12:{fopt = 2700;break;}
                }
                globalbestinit = false;
                initfinished = false;
                LastFEcount = 0;
                NFEval = 0;
                Optimizer OptZ;
                ////////////////popSize     NVars  func     Run  memory    arch size
                OptZ.Initialize(POP_SIZE, dim,func_num,RunN, mem_size_mult*dim, arch_size_mult, initial_CR, initial_F, min_pop_size, arch_prob,
                            per_of_the_best_considered_at_start, per_of_the_best_considered_at_end, norm_sigma, cauchy_sigma, default_CR, default_F,
                            MWLp1, MWLp2, MWLm, LBC_fin, startPop);
                OptZ.MainCycle();
                OptZ.Clean();
                cout<<dim<<"D Run "<<RunN+1<<" f "<<func_num<<" global best "<<globalbest-fopt<<endl;
            }
        }
        for(int func_num=1;func_num<13;func_num++)
        {
            string cur_res_file_name = res_file_name+"_" + to_string(func_num)+"_"+to_string(dim)+ ".txt";
            ofstream resArrayFile(cur_res_file_name.c_str(), ios::trunc);
            for(int step = 0;step!=17;step++){
                for(int RunN = 0;RunN!=NMBR_OF_RUNS;RunN++)
                    resArrayFile<<ResultsArray[func_num-1][RunN][step]<<"\t";
                resArrayFile<<endl;
            }
        }
    }
	return 0;
}
